#!/usr/bin/python2.4
# checklib.py - Search the archive for unnecessary dependencies
#               caused by excess NEEDED entries in ELF objects
# Copyright (C) 2006  Christian Aichinger <Greek0@gmx.net>
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 2 of the License.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import os
import errno

import pkg_tools.pkg_tools as pkg_tools
from checklib import packageinfo, db_backend, do_check, hacks, exceptions, flags, micro
from config import *

def packageinfo_from_meta(pkg):
    name = pkg["Package"]
    ver = pkg["Version"]
    file = os.path.join(Archive, pkg["Filename"])
    dir = os.path.join(Scratch, "%s_%s" % (name, ver))

    return packageinfo.PackageInfo(name, ver, file, dir)

def status_msg(str):
    print str

def store_result(pkg_meta, RES, FLAGS, unneeded = None):
    ures = micro.uResult.from_Results(
            pkg_meta["Package"],
            pkg_meta["Version"],
            RES, FLAGS, unneeded)

    db_backend.update_result(ures)

    if FLAGS & flags.FLG_ELF_IN_ARCH_ALL:
        print "Waaaaaahhhhhh. The package is arch:all and contains ELF files"
        print "It is very likely that this is an RC bug in the package!"
    if FLAGS & flags.FLG_EXCEPTION:
        print "An exception occurred, continuing with next package"

    if unneeded:
        print "The following dependencies are unneeded:"
        print "   ", ', '.join(unneeded)

    if RES & flags.RES_SKIP:
        print "RESULT: SKIP, no elf objects found. Count yourself lucky."
    elif RES & flags.RES_ABORT:
        print "RESULT: ERROR, issues described above"
    elif RES & flags.RES_ERR:
        print "RESULT: PROBLEMS, issues described above"
    elif RES & flags.RES_WARN:
        print "RESULT: NOTICE, issues described above"
    else:
        print "RESULT: SUCCESS, no problems found"

    status_msg("Done")

def get_libpkg_id(pkgfile, name, l = None):
    result = db_backend.get_libpkg(name)
    if result is not None:
        return result

    # If it's not in the DB create the objects manually
    # (using the supplied pkginfo if possible)
    if not l:
        l = packageinfo_from_meta(pkgfile[name])
    l.get_libraries()

    # And put the info into the db
    sys.stderr.write("Inserting into DB: " + l.name + '\n')
    return db_backend.add_libpkg(l)

def check_package(pkgfile, bin_pkg_name):
    RES = 0
    FLAGS = 0

    pkg_meta = pkgfile[bin_pkg_name]
    status_msg("Working on binary package '%s', version '%s'" % (bin_pkg_name, pkg_meta['Version']))
    try:
        b = packageinfo_from_meta(pkg_meta)
    except exceptions.UnpackError:
        print "Exception: Binary package '%s' could not be unpacked." % pkg_meta["Package"]
        RES |= flags.RES_ABORT
        FLAGS |= flags.FLG_CANNOT_UNPACK | flags.FLG_EXCEPTION
        return store_result(pkg_meta, RES, FLAGS)
    
    b.is_binary = True
    b.get_elf_files()

    if not b.elf_filenames:
        assert RES == 0
        return store_result(pkg_meta, flags.RES_SKIP, FLAGS)
    else:
        if pkg_meta['Architecture'].strip() == 'all':
            RES |= flags.RES_ERR
            FLAGS |= flags.FLG_ELF_IN_ARCH_ALL
            # We can savely continue here.
            # It's just a QA check and doesn't influence the checking process

    print >>sys.stderr, "Looking up needed library packages"
    b.pkg_id = bin_id = get_libpkg_id(pkgfile, bin_pkg_name, b)
    scope = [bin_id]
    for alternatives in pkg_meta["*Depends"] + pkg_meta["*Pre-Depends"]:
        for lib_pkg_name in alternatives:
            lib_pkg_name = hacks.mangle_package_name(lib_pkg_name)
            try: scope.append(get_libpkg_id(pkgfile, lib_pkg_name))
            except (KeyboardInterrupt, SystemExit): raise
            except Exception: 
                for new_name in pkgfile.provides_lookup.get(lib_pkg_name, []):
                    try: scope.append(get_libpkg_id(pkgfile, new_name))      
                    except (KeyboardInterrupt, SystemExit): raise
                    except Exception: continue
                    break
                else: continue

            #hacks.post_process_lib(lib_pkg_name_orig, scope[-1])
            #print "Selected alternative: %s out of %r" % (lib_pkg_name, alternatives)
            break
        else:
            print "Exception: No package could be instantiated for dependency '%s'" % (alternatives,)
            RES |= flags.RES_ABORT
            FLAGS |= flags.FLG_DEP_RESOLUTION | flags.FLG_EXCEPTION
            return store_result(pkg_meta, RES, FLAGS)

    print "Checking NEEDED entries"
    try:
        _res, _flags, unneeded = do_check.do_check(b, scope)
        RES |= _res
        FLAGS |= _flags
    except (NotImplementedError, exceptions.CheckError), error:
        print "Exception:", error
        RES |= flags.RES_ABORT
        FLAGS |= flags.FLG_DO_CHECK_FAIL | flags.FLG_EXCEPTION
        return store_result(pkg_meta, RES, FLAGS)

    return store_result(pkg_meta, RES, FLAGS, unneeded)


##############################
### UNINTERESTING GROUNDWORK
##############################

class TempDir:
    def __init__(self, name):
        self.name = name
    def __del__(self):
        os.spawnlp(os.P_WAIT, "rm", "rm", "-rf", self.name)

class OutputWrapper:
    def __init__(self, output_file, mode='w'):
        self.output_file = output_file
        self.f = file(output_file, mode)

        self.stored_out = sys.stdout
        sys.stdout = self

    def write(self, str):
        self.stored_out.write(str)
        self.f.write(str)

    def quit(self):
        self.f.close()
        if sys:
            sys.stdout = self.stored_out

    def __del__(self):
        self.f.close()
        if sys and sys.stdout == self:
            sys.stdout = self.stored_out

def open_packages_file():
    try:
        pkgfile = pkg_tools.PackagesFile(pkg_file_name)
    except IOError:
        tmppath = os.path.join(Cache, 'Packages')
        os.system("cp '%s' '%s'; gunzip -f '%s';" % 
                  (pkg_file_name+'.gz', tmppath+'.gz', tmppath+'.gz'))
        pkgfile = pkg_tools.PackagesFile(tmppath)

    return pkgfile

def print_error(msg):
    file(os.path.join(Logdir, 'error.txt'), 'a').write(msg+'\n')
    print msg
    pass

def run_test(package_names):
    try:
        os.mkdir(Logdir)
    except OSError, (errnumber, errstr):
        if errnumber != errno.EEXIST:
            raise

    #do_full_log = OutputWrapper(os.path.join(Logdir, 'full_log.txt'), mode='a')
    
    status_msg("Parsing Packages file")
    pkgfile = open_packages_file()

    status_msg("Initial setup")
    db_backend.init_database(Database)
    tmpdir = TempDir(Scratch)
            
    for pkgname in package_names:
        sys.stdout.write('\n\n')
        if not pkgname in pkgfile:
            print_error("'%s' not found in Packages file" % pkgname)
            continue

        pkg = pkgfile[pkgname]
        logfile = os.path.join(Logdir, 'log.%s_%s.txt' % (pkg['Package'], pkg['Version']))

        do_log = OutputWrapper(logfile)
        check_package(pkgfile, pkgname)
        do_log.quit()

def filter_strip(generator):
    for line in generator:
        yield line.strip()

def main():
    if sys.argv[1] == '--stdin':
        run_test(filter_strip(sys.stdin))
    elif sys.argv[1] == '--from-db':
        db_backend.init_database(Database)
        run_test(db_backend.get_check_queue())
    elif sys.argv[1] == '--update-pkg':
        db_backend.init_database(Database)
        pkgfile = open_packages_file()
        db_backend.update_packages_list(pkgfile)
    else:
        run_test(sys.argv[1:])

if __name__ == '__main__':
    main()
