#!/usr/bin/python
# elf.py - Quick ELF type checking functions
#          Used by checklib.py and friends.
# Copyright (C) 2006  Christian Aichinger <Greek0@gmx.net>
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 2 of the License.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import os
import stat

EI_NIDENT = 16
EI_MAG = "\x7fELF"
EI_CLASS = 4
EI_DATA = 5

ELFCLASS32 = 1
ELFCLASS64 = 2

ELFDATA2LSB = 1
ELFDATA2MSB = 2

ET_NONE = 0
ET_REL = 1
ET_EXEC = 2
ET_DYN = 3
ET_CORE = 4

def is_elf(filename):
    try:
        hdr = file(filename).read(EI_NIDENT+2)
    except:
        return False

    if hdr.startswith(EI_MAG):
        return True

    return False

def elf_hdr(filename):
    try:
        if not stat.S_ISREG(os.stat(filename)[stat.ST_MODE]):
            raise Exception
        hdr = file(filename).read(EI_NIDENT+2)
    except Exception:
        return None

    if not hdr.startswith(EI_MAG):
        return None

    elf_class = ord(hdr[EI_CLASS])
    elf_data = ord(hdr[EI_DATA])

    type = hdr[EI_NIDENT:]
    if elf_data == 1:
        ntype = ord(type[0]) + 0x100*ord(type[1])
    else:
        ntype = 0x100*ord(type[0]) + ord(type[1])

    return (elf_class, elf_data, ntype)

def elf_type(filename):
    res = elf_hdr(filename)
    if not res:
        return None

    cls, data, type = res

    if cls != ELFCLASS32 or data != ELFDATA2LSB:
        return None

    return type

def main():
    etype = elf_type(sys.argv[1])
    if etype is None:
        print "Not Elf"
    else:
        print "Elf, Type:", etype

if __name__ == '__main__':
    main()
