#!/usr/bin/python
# symbol.py - Simple representation of a symbol within an ELF binary/library
#             Used by checklib.py and friends.
# Copyright (C) 2006  Christian Aichinger <Greek0@gmx.net>
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 2 of the License.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import os

class GenericSym:
    _datamap_reloc = {
            1: 'rel_addr',
            2: 'rel_type' }
    _datamap_sym = {
            1: 'type',
            2: 'binding',
            3: 'visibility',
            4: 'section',
            5: 'address',
            6: 'size',
            7: 'index',
            8: 'name' }
    _datamap_versym = {
            1: 'ver_index',
            2: 'ver_type',
            3: 'ver_flags',
            4: 'ver_hidden',
            5: 'ver_name',
            6: 'ver_file' }

    def _parse_reloc(self, data):
        assert data[0] == 'REL'
        for key in self._datamap_reloc:
            setattr(self, self._datamap_reloc[key], data[key])

    def _parse_sym(self, data):
        assert data[0] == 'SYM'
        for key in self._datamap_sym:
            setattr(self, self._datamap_sym[key], data[key])

    def _parse_versym(self, data):
        if data[0] == 'VS':
            self.have_versym = True
            for key in self._datamap_versym:
                setattr(self, self._datamap_versym[key], data[key])
        elif data[0] == 'NOVS':
            self.have_versym = False
        else:
            print repr(data)
            raise RuntimeError("Symbol is neither VS nor NOVS... Error.")

    def __str__(self):
        ret = "%s '%s' in '%s', addr %s, section %s\n" % (
                str(self.__class__.__name__), 
                self.name, self.libname, self.address, self.section)
        if self.have_versym:
            ret += "  ver_name: %s\n" % (self.ver_name,)

        return ret

class ExportSym(GenericSym):
    def __init__(self, libname, symstr):
        self.libname = libname
        
        data = symstr.split(':')
        self._parse_sym(data)
        self._parse_versym(data[9:])
        self.undef = (self.section == 'SHN_UNDEF');

class RelocSym(GenericSym):
    def __init__(self, libname, symstr):
        self.libname = libname
        
        data = symstr.split(':')
        self._parse_reloc(data)
        self._parse_sym(data[3:])
        self._parse_versym(data[12:])
        self.undef = (self.section == 'SHN_UNDEF');
