/* dynamic.c - info about the dynamic section of ELF files
   This file is loosely based on readelf.c from the Red Hat elfutils
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#include <gelf.h>
#include <inttypes.h>
#include <libelf.h>
#include <elfutils/libebl.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "utils.h"
#include "output.h"

static void handle_dynamic(UElf_File *efile, UElf_Section *section) {
    section_fill_data(efile, section);
    if (!section->data) {
        warning("Can't get data for DYNAMIC section\n"); 
        return;
    }

    UElf_Section sec_str = UElf_Section_INIT;
    section_from_index(efile, section->shdr->sh_link, &sec_str);
    if (sec_str.shdr == NULL) {
        warning("Couldn't get strings for DYNAMIC section at %#0" PRIx64 "\n", section->shdr->sh_offset);
        return;
    }

    unsigned long int nentries = section->shdr->sh_size / section->shdr->sh_entsize;
    print_header("Dyn section, nentries: %lu: Addr: %#010" PRIx64 ", Offset: %#08" PRIx64 "\n",
            nentries,
	    section->shdr->sh_addr,
	    section->shdr->sh_offset,
	    (int) section->shdr->sh_link,
	    elf_strptr(efile->elf, efile->shstrndx, sec_str.shdr->sh_name));

    /*****************************/

    size_t cnt;
    for (cnt = 0; cnt < nentries; ++cnt) {
        GElf_Dyn dynmem;
        GElf_Dyn *dyn = gelf_getdyn (section->data, cnt, &dynmem);
        if (dyn == NULL)
            break;

        print_dynamic_entry(efile, section, dyn);
    }
}

bool show_dynamic (UElf_File *efile)
{
    UElf_Section section = UElf_Section_INIT;
    while (section_find(efile, SHT_DYNAMIC, &section))
        handle_dynamic(efile, &section);

    return true;
}
