/* hashtable.c - info about the hashtable (i.e. exported symbols) of ELF files
   This file is loosely based on readelf.c from the Red Hat elfutils
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#include <gelf.h>
#include <inttypes.h>
#include <libelf.h>
#include <elfutils/libebl.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "utils.h"
#include "output.h"

static void handle_hashtable (UElf_File *efile, UElf_Section *section) {
    section_fill_data(efile, section);
    if (!section->data) {
        warning("Can't get data for HASH section\n"); 
        return;
    }

    UElf_Section sec_sym = UElf_Section_INIT;
    section_from_index(efile, section->shdr->sh_link, &sec_sym);
    section_fill_data(efile, &sec_sym);


    if (sec_sym.shdr == NULL || sec_sym.data == NULL) {
        warning("Invalid symbol table at offset %#0" PRIx64 "\n", section->shdr->sh_offset);
        return;
    }
  
    char *dataname = section->shdr->sh_link ? 
                     elf_strptr(efile->elf, efile->shstrndx, sec_sym.shdr->sh_name) : "";
    print_header("Hash section [%2u] '%s' for symbol table [%2u] '%s' at offset %#0" PRIx64 "\n",
            (unsigned int) elf_ndxscn(section->scn),
  	    elf_strptr(efile->elf, efile->shstrndx, section->shdr->sh_name),
  	    (unsigned int) section->shdr->sh_link,
  	    dataname,
  	    section->shdr->sh_offset);
  
    Elf32_Word nbucket = ((Elf32_Word *) section->data->d_buf)[0];
    Elf32_Word nchain = ((Elf32_Word *) section->data->d_buf)[1];
    Elf32_Word *bucket = &((Elf32_Word *) section->data->d_buf)[2];
    Elf32_Word *chain = &((Elf32_Word *) section->data->d_buf)[2 + nbucket];
  
    UElf_VersionInfo *verinfo = verinfo_new(efile);
    for (Elf32_Word cnt = 0; cnt < nbucket; cnt++) {
        Elf32_Word inner = bucket[cnt];
        while (inner > 0 && inner < nchain) {
            UElf_Symbol info;
            resolve_symbol(efile, &sec_sym, inner, &info);

            print_sym(efile, inner, &info);
            print_verinfo(efile, inner, verinfo);
  
            inner = chain[inner];
        }
    }
    verinfo_free(verinfo);
}

void show_hashtable (UElf_File *efile)
{
    UElf_Section section = UElf_Section_INIT;
    while (section_find(efile, SHT_HASH, &section))
        handle_hashtable(efile, &section);
}
