/* objinfo.h - prototypes for printing errors/warnings/ELF information
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#ifndef __SEEN_OUTPUT_H
#define __SEEN_OUTPUT_H

/* The selected output format */
static const uint32_t fmt_machine    = 1<<0;
static const uint32_t fmt_noheaders  = 1<<1;
uint32_t output_format;

/* Info output functions */
void print_sym(UElf_File *efile, GElf_Word index, UElf_Symbol *info);
void print_reloc(UElf_File *efile, GElf_Rel *rel);
void print_verinfo(UElf_File *efile, GElf_Word index, UElf_VersionInfo *verinfo);
void print_section(UElf_File *efile, unsigned int index, GElf_Shdr *shdr);
void print_dynamic_entry(UElf_File *efile, UElf_Section *section, GElf_Dyn *dyn);

void print_verneed(UElf_File *efile, UElf_Section *section, unsigned int offset, GElf_Verneed *need);
void print_verneedaux(UElf_File *efile, UElf_Section *section, unsigned int offset, GElf_Vernaux *aux);
void print_verdef(UElf_File *efile, UElf_Section *section,
        unsigned int offset, GElf_Verdef *def, GElf_Verdaux *aux);
void print_verdefaux(UElf_File *efile, UElf_Section *section, unsigned int offset, GElf_Verdaux *aux);

void print_header(const char *format, ...);

/* Error, Warning and Debugging */
void _error(const char *file, unsigned int line, const char *format, ...)
                                __attribute__ ((format (printf, 3, 4)));
void _debug(const char *file, unsigned int line, const char *format, ...)
                                __attribute__ ((format (printf, 3, 4)));

#define warning(format, ...) fprintf(stderr, format, ## __VA_ARGS__)
#define debug(format, ...) _debug(__FILE__, __LINE__, format, ## __VA_ARGS__)
#define error(format, ...) _error(NULL, 0, format, ## __VA_ARGS__)
#define internal_error(format, ...) _error(__FILE__, __LINE__, format, ## __VA_ARGS__)


#endif // __SEEN_OUTPUT_H
