/* sparse_array.c - linked list serving as array for it's evil master objinfo
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include "sparse_array.h"

struct sarray_entry {
    struct sarray_entry *next;
    sa_key_t key;
    void* value;
};

struct sarray_head {
    struct sarray_entry *head;
};


// New & Free
sarray sarray_new() {
    sarray a = malloc(sizeof(struct sarray_head));
    a->head = NULL;
    return a;
}

static void sarray_free_entries(sarray_entry *entry) {
    sarray_entry *next = entry->next;
    free(entry);
    if (next)
        sarray_free_entries(next);
}

void sarray_free(sarray a) {
    if (a->head)
        sarray_free_entries(a->head);
    free(a);
}

// First, Next & Count
sarray_entry *sarray_first(sarray a) {
    return a->head;
}

sarray_entry *sarray_next(sarray_entry *entry) {
    if (entry)
        return entry->next;
    return NULL;
}

unsigned long int sarray_count(sarray a) {
    sarray_entry *entry = a->head;
    unsigned long int res = 0;

    while (entry) {
        res++;
        entry = entry->next;
    }

    return res;
}

// Key & Value
sa_key_t sarray_key(sarray_entry *entry) {
    return entry->key;
}

void* sarray_value(sarray_entry *entry) {
    return entry->value;
}

// Insert
void sarray_insert(sarray a, sa_key_t key, void* value) {
    if (sarray_find(a, key)) {
        fprintf(stderr, "Fuck. Key alraedy exists\n");
        return;
    }

    sarray_entry *entry = malloc(sizeof(sarray_entry));

    entry->next = a->head;
    entry->key = key;
    entry->value = value;

    a->head = entry;
}

// Find
void *sarray_find(sarray a, sa_key_t key) {
    sarray_entry *entry = a->head;

    while (entry) {
        if (entry->key == key)
            return entry->value;

        entry = entry->next;
    }

    return NULL;
}
